#!/usr/bin/env python
# coding: utf-8

#THIS SCRIPT DEPENDS ON PYTHON 2.9, ARCPY, THE 3D/SPATIAL ANALYST EXTENSIONS
#AND PANDAS. 
#THIS SCRIPT REQUIRES BUILDING OBJECTS AND USER DEFINED PATHWAYS.
#THE OUTPUTDIR MUST BE THE SAME AS IN GETVEGETATIONMETRICS_V3 WITH "\final" ADDED
#THIS SCRIPT JOINS THE VEG METRICS TO THE BUILDING OBJECTS BY BLDG_ID
#CONTACT: ZACKARY WERNER (ZACKARY.WERNER@USDA.GOV)

# Load libraries
import os
import pandas as pd
import arcpy
import glob
# import arcgis
#from arcpy.sa import *

arcpy.CheckOutExtension("spatial")
arcpy.CheckOutExtension("3D")
arcpy.env.overwriteOutput = True
print("libs")

#########################
# USER DEFINED PERAMETERS
#########################

# Set Parameters
outputDir = r'F:\WUI\tubbs\updated\final'

# Input Data
in_shp = r'F:\WUI\tubbs\buildings\tubbsLidarBuildingsFinal.shp'

gdb = r"F:\WUI\tubbs\tubbsLocal.gdb"

arcpy.env.workspace = gdb

#########################
# USER DEFINED PERAMETERS
#########################

# Prep the output dir
if not os.path.exists(os.path.join(outputDir, "working")):
    os.makedirs(os.path.join(outputDir, "working"))

if not os.path.exists(os.path.join(outputDir, "final")):
    os.makedirs(os.path.join(outputDir, "final"))

#pull these from the \final folder from the previous scripts

vegCsvs = glob.glob(outputDir+'\*VegZ*.csv')
chmCsvs = glob.glob(outputDir+'\*CHM*.csv')
overCsvs = glob.glob(outputDir+'\*Overhang*.csv')

print(vegCsvs)
print(chmCsvs)
print(overCsvs)

vegDf1 = pd.read_csv(vegCsvs[0])
vegDf2 = pd.read_csv(vegCsvs[1])
vegDf3 = pd.read_csv(vegCsvs[2])

vegDF = vegDf1[['BLDG_ID','perVegZ0']]
vegDF = pd.merge(vegDF, vegDf2[['BLDG_ID','perVegZ1']], on='BLDG_ID', how='outer')
vegDF = pd.merge(vegDF, vegDf3[['BLDG_ID','perVegZ2']], on='BLDG_ID', how='outer')

chmDf1 = pd.read_csv(chmCsvs[0])
chmDf2 = pd.read_csv(chmCsvs[1])
chmDf3 = pd.read_csv(chmCsvs[2])

finalDF = pd.merge(vegDF, chmDf1[['BLDG_ID','AreaZ0','Z0MinCh', 'Z0MaxCh','Z0AvgCh']], on='BLDG_ID', how='outer')
finalDF = pd.merge(finalDF, chmDf2[['BLDG_ID','AreaZ1','Z1MinCh', 'Z1MaxCh','Z1AvgCh']], on='BLDG_ID', how='outer')
finalDF = pd.merge(finalDF, chmDf3[['BLDG_ID','AreaZ2','Z2MinCh', 'Z2MaxCh','Z2AvgCh']], on='BLDG_ID', how='outer')

overDF = pd.read_csv(overCsvs[0])

finalDF = pd.merge(finalDF, overDF[['BLDG_ID','perOver']], on='BLDG_ID', how='outer')

finalDF.to_csv(os.path.join(outputDir,'working\LidarResults.csv'))


arcpy.conversion.FeatureClassToFeatureClass(in_shp, os.path.join(outputDir, 'final'), 'LidarAttributedBuildings.shp')

arcpy.management.JoinField(os.path.join(outputDir,'final\LidarAttributedBuildings.shp'),
                           'BLDG_ID', os.path.join(outputDir,'working\LidarResults.csv'),
                           'BLDG_ID', list(finalDF.columns))

print("done")

