#!/usr/bin/env python
# coding: utf-8

#THIS SCRIPT DEPENDS ON PYTHON 2.9, ARCPY AND THE 3D ANALYST EXTENSION. 
#THIS SCRIPT REQUIRES BUILDING OBJECTS AS INPUT AND USER DEFINED PATHWAYS.
#THIS SCRIPT CREATES CAL FIRE DEFENSIBLE SPACE ZONES FOR EACH BUILDING. 
#CONTACT: ZACKARY WERNER (ZACKARY.WERNER@USDA.GOV)

# Load libraries
import os
# os.environ["CUDA_VISIBLE_DEVICES"] = "-1"
# import pandas as pd
import arcpy

# import arcgis
arcpy.CheckOutExtension("3D")
arcpy.env.overwriteOutput = True
print("libs")

#########################
# USER DEFINED PERAMETERS
#########################

# Set Parameters 
outputDir = r'F:\WUI\tubbs'

# Input Data
in_shp = r'F:\WUI\tubbs\buildings\tubbsLidarBuildingsFinal.shp'

# the projection you want the final product in
proj = 'PROJCS["NAD_1983_2011_StatePlane_California_II_FIPS_0402_Ft_US",GEOGCS["GCS_NAD_1983_2011",DATUM["D_NAD_1983_2011",SPHEROID["GRS_1980",6378137.0,298.257222101]],PRIMEM["Greenwich",0.0],UNIT["Degree",0.0174532925199433]],PROJECTION["Lambert_Conformal_Conic"],PARAMETER["false_easting",6561666.666666666],PARAMETER["false_northing",1640416.666666667],PARAMETER["central_meridian",-122.0],PARAMETER["standard_parallel_1",38.33333333333334],PARAMETER["standard_parallel_2",39.5],PARAMETER["latitude_of_origin",37.66666666666666],UNIT["Foot_US",0.3048006096012192]],VERTCS["NAVD88 height",VDATUM["North American Vertical Datum 1988"],PARAMETER["Vertical_Shift",0.0],PARAMETER["Direction",1.0],UNIT["metre",1.0]]'

##########################
# USER DEFINED PERAMETERS
#########################

# Prep the output dir
if not os.path.exists(os.path.join(outputDir, "working")):
    os.makedirs(os.path.join(outputDir, "working"))

if not os.path.exists(os.path.join(outputDir, "final")):
    os.makedirs(os.path.join(outputDir, "final"))

#########################
# ADD BUILDING ID'S HERE (FIELD CALCULATOR)
#########################
arcpy.AddField_management(in_shp, "BLDG_ID", "LONG")

startNumber = 0

with arcpy.da.UpdateCursor(in_shp, "BLDG_ID") as cursor:
    for row in cursor:
        row[0] = startNumber
        startNumber = startNumber + 1
        cursor.updateRow(row)

fields = [f.name for f in arcpy.ListFields(in_shp)]
print(fields)

print("creating zone 0")
with arcpy.EnvManager(parallelProcessingFactor="100%"):
    buffer5 = arcpy.analysis.Buffer(in_shp, os.path.join(outputDir, 'working\BuildingsBuffer5.shp'), "5 Feet", "FULL",
                                    "ROUND", "NONE", None, "PLANAR")

with arcpy.EnvManager(parallelProcessingFactor="100%"):
    zone0 = arcpy.analysis.Buffer(in_shp, os.path.join(outputDir, 'final\BuildingsZone0.shp'), "5 Feet", "OUTSIDE_ONLY",
                                  "ROUND", "NONE", None, "PLANAR")

print("creating zone 1")
with arcpy.EnvManager(parallelProcessingFactor="100%"):
    buffer30 = arcpy.analysis.Buffer(buffer5, os.path.join(outputDir, 'working\BuildingsBuffer30.shp'), "30 Feet",
                                     "FULL", "ROUND", "NONE", None, "PLANAR")

with arcpy.EnvManager(parallelProcessingFactor="100%"):
    zone1 = arcpy.analysis.Buffer(buffer5, os.path.join(outputDir, 'final\BuildingsZone1.shp'), "30 Feet",
                                  "OUTSIDE_ONLY", "ROUND", "NONE", None, "PLANAR")

# with arcpy.EnvManager(parallelProcessingFactor="100%"):
#     zone1 = arcpy.analysis.SymDiff(buffer5, buffer30, os.path.join(outputDir,'working\BuildingsZone1.shp'), "ALL", None)

print("creating zone 2")
with arcpy.EnvManager(parallelProcessingFactor="100%"):
    zone2 = arcpy.analysis.Buffer(buffer30, os.path.join(outputDir, 'final\BuildingsZone2.shp'), "100 Feet",
                                  "OUTSIDE_ONLY", "ROUND", "NONE", None, "PLANAR")

arcpy.management.CalculateField(zone0, "ZONE", 0, "PYTHON3", '', "SHORT")
arcpy.management.CalculateField(zone1, "ZONE", 1, "PYTHON3", '', "SHORT")
arcpy.management.CalculateField(zone2, "ZONE", 2, "PYTHON3", '', "SHORT")

print("done")
