#!/usr/bin/env python
# coding: utf-8

#THIS SCRIPT DEPENDS ON PYTHON 2.9, ARCPY AND THE 3D/SPATIAL ANALYST EXTENSIONS. 
#THIS SCRIPT REQUIRES BUILDING OBJECTS, DEFENSIBLE SPACE ZONES,
#LIDAR DATA AS INPUT AND USER DEFINED PATHWAYS.
#THIS SCRIPT CREATES A DTM, DSM, CHM AND BINARY CANOPY COVER RASTERS 
#FROM A LIDAR POINT CLOUD AND CALCULATES ZONAL STATISTICS FOR EACH DEFENSIBLE
#SPACE ZONE FOR EACH BUILDING.
#CONTACT: ZACKARY WERNER (ZACKARY.WERNER@USDA.GOV)

# Load libraries
import os
# os.environ["CUDA_VISIBLE_DEVICES"] = "-1"
import pandas as pd
import arcpy
import time

# import arcgis
arcpy.CheckOutExtension("3D")
arcpy.CheckOutExtension("Spatial")
from arcpy.sa import *

arcpy.env.overwriteOutput = True
print("libs")

st = time.time()

#############
# START USER INPUT
#############
outputDir = r'F:\WUI\tubbs\updated'

# Input Data
buildings = r'F:\WUI\tubbs\buildings\tubbsLidarBuildingsFinal.shp'

zones = [r'F:\WUI\tubbs\final\BuildingsZone0.shp',
         r'F:\WUI\tubbs\final\BuildingsZone1.shp',
         r'F:\WUI\tubbs\final\BuildingsZone2.shp']

in_lasd = r'F:\WUI\tubbs\lidar\tubbsLidarClipped.lasd'

gdb = r"F:\WUI\tubbs\tubbsPrefireScript.gdb"
arcpy.env.workspace = gdb

res = 3.28 #6.56 survey ft = 2 meters

#proj = "PROJCS['NAD_1983_HARN_StatePlane_Colorado_Central_FIPS_0502_Feet',GEOGCS['GCS_North_American_1983_HARN',DATUM['D_North_American_1983_HARN',SPHEROID['GRS_1980',6378137.0,298.257222101]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]],PROJECTION['Lambert_Conformal_Conic'],PARAMETER['False_Easting',3000000.000316083],PARAMETER['False_Northing',999999.999996],PARAMETER['Central_Meridian',-105.5],PARAMETER['Standard_Parallel_1',38.45],PARAMETER['Standard_Parallel_2',39.75],PARAMETER['Latitude_Of_Origin',37.83333333333334],UNIT['Foot_US',0.3048006096012192]],VERTCS['NAVD88 - Geoid96 (Feet)',VDATUM['unknown'],PARAMETER['Vertical_Shift',0.0],PARAMETER['Direction',1.0],UNIT['Foot_US',0.3048006096012192]]"
#proj = "PROJCS['NAD_1983_2011_StatePlane_Tennessee_FIPS_4100_Ft_US',GEOGCS['GCS_NAD_1983_2011',DATUM['D_NAD_1983_2011',SPHEROID['GRS_1980',6378137.0,298.257222101]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]],PROJECTION['Lambert_Conformal_Conic'],PARAMETER['False_Easting',1968500.0],PARAMETER['False_Northing',0.0],PARAMETER['Central_Meridian',-86.0],PARAMETER['Standard_Parallel_1',35.25],PARAMETER['Standard_Parallel_2',36.41666666666666],PARAMETER['Latitude_Of_Origin',34.33333333333334],UNIT['Foot_US',0.3048006096012192]],VERTCS['NAVD88 height - Geoid12B (ftUS)',VDATUM['North American Vertical Datum 1988'],UNIT['US survey foot',0.3048006096012192]]"
proj = 'PROJCS["NAD_1983_2011_StatePlane_California_II_FIPS_0402_Ft_US",GEOGCS["GCS_NAD_1983_2011",DATUM["D_NAD_1983_2011",SPHEROID["GRS_1980",6378137.0,298.257222101]],PRIMEM["Greenwich",0.0],UNIT["Degree",0.0174532925199433]],PROJECTION["Lambert_Conformal_Conic"],PARAMETER["false_easting",6561666.666666666],PARAMETER["false_northing",1640416.666666667],PARAMETER["central_meridian",-122.0],PARAMETER["standard_parallel_1",38.33333333333334],PARAMETER["standard_parallel_2",39.5],PARAMETER["latitude_of_origin",37.66666666666666],UNIT["Foot_US",0.3048006096012192]],VERTCS["NAVD88 height",VDATUM["North American Vertical Datum 1988"],PARAMETER["Vertical_Shift",0.0],PARAMETER["Direction",1.0],UNIT["metre",1.0]]'

#############
# END USER INPUT
#############

# Prep the output directories
if not os.path.exists(os.path.join(outputDir, "working")):
    os.makedirs(os.path.join(outputDir, "working"))

if not os.path.exists(os.path.join(outputDir, "final")):
    os.makedirs(os.path.join(outputDir, "final"))

#STEP 1: FILTER LASD WITH MAKE LAS DATASET LAYER
#NOTE: CREATING THE LIDAR POINT CLOUD SUBSETS PROGRAMATICALLY YIELDED 
#SLIGHTLY DIFFERENT RESULTS THAN IN THE GUI. THE FINAL RESULTS FOR THE 
#WUI GEOTASC PROJECT WERE CREATED FROM POINT CLOUDS SUBSET IN THE ARC PRO.

#make las dataset layer GROUND ONLY
groundLasd = arcpy.management.MakeLasDatasetLayer(in_lasd,
                                                  "groundLasd",
                                                  "2",
                                                  None,
                                                  "INCLUDE_UNFLAGGED",
                                                  "INCLUDE_SYNTHETIC",
                                                  "INCLUDE_KEYPOINT",
                                                  "EXCLUDE_WITHHELD",
                                                  None,
                                                  "INCLUDE_OVERLAP")

#make las dataset layer NO BUILDINGS
noBuildingsLasd = arcpy.management.MakeLasDatasetLayer(in_lasd,
                                                       "noBuildingsLasd",
                                                       "1;2;3;4;5",
                                                       "1", #firstHit
                                                       "INCLUDE_UNFLAGGED",
                                                       "INCLUDE_SYNTHETIC",
                                                       "INCLUDE_KEYPOINT",
                                                       "EXCLUDE_WITHHELD",
                                                       None,
                                                       "INCLUDE_OVERLAP")

buildingsLasd = arcpy.management.MakeLasDatasetLayer(in_lasd,
                                                  "buildingsLasd",
                                                  "6",
                                                  None,
                                                  "INCLUDE_UNFLAGGED",
                                                  "INCLUDE_SYNTHETIC",
                                                  "INCLUDE_KEYPOINT",
                                                  "EXCLUDE_WITHHELD",
                                                  None,
                                                  "INCLUDE_OVERLAP")

print("creating dtm")
#STEP 2: CREATE DTM AND DSM RASTERS FROM FILTERED LASD
dtm = arcpy.conversion.LasDatasetToRaster(groundLasd,
                                        "dtm_script_zw",
                                        "ELEVATION",
                                        "BINNING AVERAGE LINEAR",
                                        "FLOAT",
                                        "CELLSIZE", res, res)

print("creating dsm")
dsm = arcpy.conversion.LasDatasetToRaster(noBuildingsLasd,
                                        "dsm_script_zw",
                                        "ELEVATION",
                                        "BINNING MAXIMUM SIMPLE",
                                        "FLOAT",
                                        "CELLSIZE", res, res)


#STEP 3: CREATE CHM
print("creating chm")
chm = arcpy.ddd.Minus(dsm,
                      dtm,
                      "chm_script_zw")

print("creating building raster")
#STEP 4: CREATE BUILDING RASTER FOR MASK
buildingRaster = arcpy.conversion.LasDatasetToRaster(buildingsLasd,
                                                     "buildingRaster",
                                                     "ELEVATION",
                                                     "BINNING AVERAGE SIMPLE",
                                                     "FLOAT",
                                                     "CELLSIZE",
                                                     res, res)

buildingRasterReclass = Reclassify(buildingRaster,
                                     "Value",
                                     RemapRange([[0,100000,-1]]))

print("creating mosaic to mask buildings")
#STEP 5: USE BUILDING RASTER TO MASK BUILDINGS IN CHM
rastList = [chm, buildingRasterReclass]
mosaic = arcpy.management.MosaicToNewRaster(rastList,
                                            gdb,
                                            "chm_building_mosaic",
                                            proj,
                                            "32_BIT_FLOAT",
                                            res,
                                            1,
                                            "MINIMUM",
                                            "FIRST")

print("creating masked chm raster")
#STEP 6: CREATE FINAL CHM REMOVING ALL VALUES < 2 TO MAP ONLY VEG
chm_final = arcpy.ia.SetNull(mosaic,
                             mosaic,
                             "VALUE < 6.56")

chm_final.save(os.path.join(gdb, "chm_final"))

print("creating binary veg raster")
#STEP 7: CREATE FINAL VEG RASTER SETTING CHM VALUES TO 1 AND NULL VALUES TO 0
vegRaster = Reclassify(chm_final,
                       "Value",
                       RemapRange([[0,100000,1]]))

zero = 0
veg_final = Con(IsNull(vegRaster), zero, vegRaster)
veg_final.save(os.path.join(gdb, "veg_raster_final"))

#82 seconds

print("downsampling chm and veg rasters for analysis")
#STEP 8: DOWNSAMPLE VEG AND CHM RASTERS FOR ZONAL STATS
chmResample = arcpy.management.Resample(chm_final,
                                        os.path.join(gdb, "chm_raster_final_1m"),
                                        "1 1",
                                        "NEAREST")

vegResample = arcpy.management.Resample(veg_final,
                                        os.path.join(gdb, "veg_raster_final_1m"),
                                        "1 1",
                                        "NEAREST")

#194s seconds from start

#STEP 9: GET VEG ZONAL STATS FOR CAL FIRE DSPACE ZONES
print("getting veg zonal stats")
for x in range(3):
    print(x)
    in_shp = zones[x]
    arcpy.sa.TabulateArea(in_shp, "BLDG_ID", vegResample, "Value",
                          os.path.join(outputDir, 'working\TabulateArea_DSpaceZone{}.dbf'.format(x)))
    arcpy.conversion.TableToTable(os.path.join(outputDir, 'working\TabulateArea_DSpaceZone{}.dbf'.format(x)),
                                  os.path.join(outputDir, "working"),
                                  "TabulateArea_DSpaceZone{}.csv".format(x))
    pdf = pd.read_csv(os.path.join(outputDir, 'working\TabulateArea_DSpaceZone{}.csv'.format(x)))

    # pdf.filter(like='VALUE')

    pdf['Sum'] = pdf.filter(like='VALUE').sum(axis=1)

    pdf['perVegZ{}'.format(x)] = ((pdf['VALUE_1'] / pdf['Sum']) * 100).round(2)

    pdf.to_csv(os.path.join(outputDir, 'final\VegZone{}.csv'.format(x)))

print("done")

del(pdf)

#COMMENTED OUT BECAUSE THE VEG RASTER IS CREATED IN A WAY THAT THIS DOESN'T WORK
#I'M LEAVING IT IN HERE IN CASE MORE WORK IS DONE TO GET AN APPROPRIATE VEG 
#RASTER THEN SOMETHING LIKE THESE LINES OF CODE COULD HELP GUIDE FUTURE WORK
#STEP 10: GET VEG ZONAL STATS FOR BUILDING OVERHANG
# =============================================================================
# print("getting overhang zonal stats")
# arcpy.sa.TabulateArea(buildings, "BLDG_ID", vegResample, "Value", os.path.join(outputDir,'working\TabulateArea_overhang.dbf'))
# 
# arcpy.conversion.TableToTable(os.path.join(outputDir,'working\TabulateArea_overhang.dbf'),
#                                   os.path.join(outputDir, "working"),
#                                   "TabulateArea_overhang.csv")
# pdf = pd.read_csv(os.path.join(outputDir,'working\TabulateArea_overhang.csv'))
# 
# #pdf.filter(like='VALUE')
# 
# pdf['Sum'] = pdf.filter(like='VALUE').sum(axis=1)
# 
# pdf['perOver'] = ((pdf['VALUE_1'] / pdf['Sum'])*100).round(2)
# 
# pdf.to_csv(os.path.join(outputDir,'final\BldgOverhang.csv'))
# print("done")
# 
# del(pdf)
# 
# =============================================================================
#STEP 11: GET VEG ZONAL STATS FOR CHM

print("getting chm zonal statistics")
with arcpy.EnvManager(parallelProcessingFactor="100%"):
     dbf0 = arcpy.ia.ZonalStatisticsAsTable(zones[0], "BLDG_ID", chmResample, os.path.join(outputDir, "working/ZStatsCHM0.dbf"),
                                    "DATA", "MIN_MAX_MEAN", "CURRENT_SLICE", 90)#UTO_DETECT")
with arcpy.EnvManager(parallelProcessingFactor="100%"):
     dbf1 = arcpy.ia.ZonalStatisticsAsTable(zones[1], "BLDG_ID", chmResample, os.path.join(outputDir, "working/ZStatsCHM1.dbf"),
                                    "DATA", "MIN_MAX_MEAN", "CURRENT_SLICE", 90)#, "AUTO_DETECT")
with arcpy.EnvManager(parallelProcessingFactor="100%"):
     dbf2 = arcpy.ia.ZonalStatisticsAsTable(zones[2], "BLDG_ID", chmResample, os.path.join(outputDir, "working/ZStatsCHM2.dbf"),
                                    "DATA", "MIN_MAX_MEAN", "CURRENT_SLICE", 90)#, "AUTO_DETECT")

#dbf = os.path.join(outputDir, "final/ZStatsCHM.dbf")

zstat0 = arcpy.conversion.TableToTable(dbf0, gdb, "ZStatsCHM0")
zstat1 = arcpy.conversion.TableToTable(dbf1, gdb, "ZStatsCHM1")
zstat2 = arcpy.conversion.TableToTable(dbf2, gdb, "ZStatsCHM2")

#############
#RENAME FIELDS NOT WORKING
#MAYBE rename dbf0,1,2 or write zstat0,1,2 as new table...?
#############

print("renaming fields")
namesDict0 = {"MIN": "Z0MinCh", "MAX": "Z0MaxCh", "MEAN": "Z0AvgCh", "AREA": "AreaZ0"}
namesDict1 = {"MIN": "Z1MinCh", "MAX": "Z1MaxCh", "MEAN": "Z1AvgCh", "AREA": "AreaZ1"}
namesDict2 = {"MIN": "Z2MinCh", "MAX": "Z2MaxCh", "MEAN": "Z2AvgCh", "AREA": "AreaZ2"}

for fieldName in namesDict0:
        arcpy.AlterField_management(zstat0, fieldName, namesDict0[fieldName])

for fieldName in namesDict1:
        arcpy.AlterField_management(zstat1, fieldName, namesDict1[fieldName])

for fieldName in namesDict2:
        arcpy.AlterField_management(zstat2, fieldName, namesDict2[fieldName])


zstat0 = arcpy.conversion.TableToTable(zstat0, os.path.join(outputDir, "final"), "ZStatsCHM0.csv")
zstat1 = arcpy.conversion.TableToTable(zstat1, os.path.join(outputDir, "final"), "ZStatsCHM1.csv")
zstat2 = arcpy.conversion.TableToTable(zstat2, os.path.join(outputDir, "final"), "ZStatsCHM2.csv")

print("done with all zonal stats")

et = time.time()

tt = et-st
print(tt)

